ZkStdLib

Struct ZkStdLib 

pub struct ZkStdLib { /* private fields */ }
The ZkStdLib exposes all tools that are used in circuit generation.

Implementations

Source

impl ZkStdLib

pub fn new(config: &ZkStdLibConfig, max_bit_len: usize) -> Self

Creates a new ZkStdLib given its config.

pub fn configure( meta: &mut ConstraintSystem<Fq>, (arch, max_bit_len): (ZkStdLibArch, u8), ) -> ZkStdLibConfig

Configure ZkStdLib from scratch.

Source

impl ZkStdLib

pub fn jubjub(&self) -> &EccChip<JubjubExtended>

Native EccChip.

pub fn biguint( &self, ) -> &BigUintGadget<Fq, NativeGadget<Fq, P2RDecompositionChip<Fq>, NativeChip<Fq>>>

Gadget for performing in-circuit big-unsigned integer operations.

pub fn map_gadget( &self, ) -> &MapGadget<Fq, NativeGadget<Fq, P2RDecompositionChip<Fq>, NativeChip<Fq>>, PoseidonChip<Fq>>

Gadget for performing map and non-map checks

pub fn secp256k1( &self, ) -> &ForeignWeierstrassEccChip<Fq, K256, MultiEmulationParams, FieldChip<Fq, Fq, MultiEmulationParams, NativeGadget<Fq, P2RDecompositionChip<Fq>, NativeChip<Fq>>>, NativeGadget<Fq, P2RDecompositionChip<Fq>, NativeChip<Fq>>>

Chip for performing in-circuit operations over the Secp256k1 curve, its scalar field or its base field.

pub fn p256( &self, ) -> &ForeignWeierstrassEccChip<Fq, P256, MultiEmulationParams, FieldChip<Fq, Fq, MultiEmulationParams, NativeGadget<Fq, P2RDecompositionChip<Fq>, NativeChip<Fq>>>, NativeGadget<Fq, P2RDecompositionChip<Fq>, NativeChip<Fq>>>

Chip for performing in-circuit operations over the P256 curve, its scalar field or its base field.

pub fn bls12_381( &self, ) -> &ForeignWeierstrassEccChip<Fq, G1Projective, G1Projective, NativeGadget<Fq, P2RDecompositionChip<Fq>, NativeChip<Fq>>, NativeGadget<Fq, P2RDecompositionChip<Fq>, NativeChip<Fq>>>

Chip for performing in-circuit operations over the BLS12-381 curve, its scalar field or its base field.

pub fn curve25519( &self, ) -> &ForeignEdwardsEccChip<Fq, Curve25519, MultiEmulationParams, FieldChip<Fq, Scalar, MultiEmulationParams, NativeGadget<Fq, P2RDecompositionChip<Fq>, NativeChip<Fq>>>, NativeGadget<Fq, P2RDecompositionChip<Fq>, NativeChip<Fq>>>

Chip for performing in-circuit operations over Curve25519.

pub fn base64(&self) -> &Base64Chip<Fq>

Chip for performing in-circuit base64 decoding.

pub fn parser( &self, ) -> &ParserGadget<Fq, NativeGadget<Fq, P2RDecompositionChip<Fq>, NativeChip<Fq>>>

Gadget for column-free parsing helpers (fetch_bytes, date_to_int, etc.). Always available (no arch flag needed).

pub fn scanner(&self) -> &ScannerChip<Fq>

Chip for various scanning functions based on lookups. This includes automaton-based parsing (ScannerChip::parse) and substring checks (ScannerChip::check_subsequence, ScannerChip::check_bytes).

Returns the scanner chip for automaton-based parsing and substring checks. The static automaton table is loaded automatically when parse is called with a Static(..) variant.

pub fn verifier(&self) -> &VerifierGadget<BlstrsEmulation>

Chip for performing in-circuit verification of proofs (generated with Poseidon as the Fiat-Shamir transcript hash).

pub fn assert_true( &self, layouter: &mut impl Layouter<Fq>, input: &AssignedBit<Fq>, ) -> Result<(), Error>

Assert that a given assigned bit is true.

let input: AssignedBit<F> = chip.assign_fixed(layouter, true)?;
chip.assert_true(layouter, &input)?;
pub fn assert_false( &self, layouter: &mut impl Layouter<Fq>, input: &AssignedBit<Fq>, ) -> Result<(), Error>

Assert that a given assigned bit is false

pub fn lower_than( &self, layouter: &mut impl Layouter<Fq>, x: &AssignedNative<Fq>, y: &AssignedNative<Fq>, n: u32, ) -> Result<AssignedBit<Fq>, Error>

Returns 1 iff x < y.

let x: AssignedNative<F> = chip.assign_fixed(layouter, F::from(127))?;
let y: AssignedNative<F> = chip.assign_fixed(layouter, F::from(212))?;
let condition = chip.lower_than(layouter, &x, &y, 8)?;

chip.assert_true(layouter, &condition)?;
Unsatisfiable Circuit

If x or y are not in the range [0, 2^n).

let x: AssignedNative<F> = chip.assign_fixed(layouter, F::from(127))?;
let y: AssignedNative<F> = chip.assign_fixed(layouter, F::from(212))?;
let _condition = chip.lower_than(layouter, &x, &y, 7)?;

Setting n > (F::NUM_BITS - 1) / 2 will result in a compile-time error.

pub fn poseidon( &self, layouter: &mut impl Layouter<Fq>, input: &[AssignedNative<Fq>], ) -> Result<AssignedNative<Fq>, Error>

Poseidon hash from a slice of native values into a native value.

let x: AssignedNative<F> = chip.assign_fixed(layouter, F::from(127))?;
let y: AssignedNative<F> = chip.assign_fixed(layouter, F::from(212))?;

let _hash = chip.poseidon(layouter, &[x, y])?;
pub fn poseidon_varlen<const M: usize>( &self, layouter: &mut impl Layouter<Fq>, input: &AssignedVector<Fq, AssignedNative<Fq>, M, RATE>, ) -> Result<AssignedNative<Fq>, Error>

Poseidon hash over a payload whose element count can vary between proofs.

Unlike poseidon, which takes a plain slice whose length is structurally fixed in the circuit (the same for every proof), this variant takes an AssignedVector: a specialized structure for carrying data of varying length, up to a maximum capacity of M elements.

M must be a multiple of 2 (the Poseidon absorption rate).

pub fn hash_to_curve( &self, layouter: &mut impl Layouter<Fq>, inputs: &[AssignedNative<Fq>], ) -> Result<AssignedNativePoint<JubjubExtended>, Error>

Hashes a slice of assigned values into (x, y) coordinates which are guaranteed to be in the curve C. For usage, see HashToCurveGadget.

pub fn sha2_256( &self, layouter: &mut impl Layouter<Fq>, input: &[AssignedByte<Fq>], ) -> Result<[AssignedByte<Fq>; 32], Error>

SHA2-256. Takes as input a slice of assigned bytes and returns the assigned input/output in bytes. We assume the field uses little endian encoding.

let input = chip.assign_many(
    layouter,
    &[
        Value::known(13),
        Value::known(226),
        Value::known(119),
        Value::known(5),
    ],
)?;

let _hash = chip.sha2_256(layouter, &input)?;
pub fn sha2_256_varlen<const M: usize>( &self, layouter: &mut impl Layouter<Fq>, input: &AssignedVector<Fq, AssignedByte<Fq>, M, 64>, ) -> Result<[AssignedByte<Fq>; 32], Error>

SHA-256 hash over a payload whose byte count can vary between proofs.

Unlike sha2_256, which takes a plain slice whose length is structurally fixed in the circuit (the same for every proof), this variant takes an AssignedVector: a specialized structure for carrying data of varying length, up to a maximum capacity of M bytes.

M must be a multiple of 64 (the SHA-256 block size).

pub fn sha2_512( &self, layouter: &mut impl Layouter<Fq>, input: &[AssignedByte<Fq>], ) -> Result<[AssignedByte<Fq>; 64], Error>

SHA2-512 hash.

pub fn sha3_256( &self, layouter: &mut impl Layouter<Fq>, input: &[AssignedByte<Fq>], ) -> Result<[AssignedByte<Fq>; 32], Error>

SHA3-256 hash (third-party implementation).

pub fn keccak_256( &self, layouter: &mut impl Layouter<Fq>, input: &[AssignedByte<Fq>], ) -> Result<[AssignedByte<Fq>; 32], Error>

Keccak-256 hash (third-party implementation).

pub fn blake2b_256( &self, layouter: &mut impl Layouter<Fq>, input: &[AssignedByte<Fq>], ) -> Result<[AssignedByte<Fq>; 32], Error>

Blake2b hash with a 256-bit output, unkeyed (third-party implementation).

pub fn blake2b_512( &self, layouter: &mut impl Layouter<Fq>, input: &[AssignedByte<Fq>], ) -> Result<[AssignedByte<Fq>; 64], Error>

Blake2b hash with a 512-bit output, unkeyed (third-party implementation).