ZkStdLibArch

Struct ZkStdLibArch 

pub struct ZkStdLibArch {
Show 14 fields pub jubjub: bool, pub poseidon: bool, pub sha2_256: bool, pub sha2_512: bool, pub keccak_256: bool, pub sha3_256: bool, pub blake2b: bool, pub secp256k1: bool, pub p256: bool, pub bls12_381: bool, pub curve25519: bool, pub base64: bool, pub automaton: bool, pub nr_pow2range_cols: u8,
}
Architecture of the standard library. Specifies what chips need to be configured.

Fields

jubjub: bool

Enable the Jubjub chip?

poseidon: bool

Enable the Poseidon chip?

sha2_256: bool

Enable the SHA256 chip?

sha2_512: bool

Enable the SHA512 chip?

keccak_256: bool

Enable the Keccak chip? (third-party implementation)

Note: is configured using the same columns and tables as sha3_256, meaning enabling either of the two, or both, requires the same configuration resources.

sha3_256: bool

Enable the Sha3 chip? (third-party implementation)

Note: is configured using the same columns and tables as keccak_256, meaning enabling either of the two, or both, requires the same configuration resources.

blake2b: bool

Enable the Blake2b chip? (third-party implementation)

secp256k1: bool

Enable the Secp256k1 chip?

p256: bool

Enable the P256 chip?

bls12_381: bool

Enable BLS12-381 chip?

curve25519: bool

Enable Curve25519 chip?

base64: bool

Enable base64 chip?

automaton: bool

Enable scanner chip (automaton-based parsing and substring checks)?

nr_pow2range_cols: u8

Number of parallel lookups for range checks.

Implementations

Source

impl ZkStdLibArch

pub fn write<W: Write>(&self, writer: &mut W) -> Result<()>

Writes the ZKStd architecture to a buffer.

pub fn read<R: Read>(reader: &mut R) -> Result<Self>

Reads the ZkStd architecture from a buffer.

pub fn read_from_serialized_vk<R: Read>(reader: &mut R) -> Result<Self>

Reads the ZkStdArchitecture from a buffer where a MidnightVK was serialized. This enables the reader to know the architecture without the need of deserializing the full verifying key.