pub fn batch_verify<H: TranscriptHash + Send + Sync>(
params_verifier: &ParamsVerifierKZG<Bls12>,
vks: &[MidnightVK],
pis: &[Vec<Fq>],
proofs: &[Vec<u8>],
) -> Result<(), Error>Expand description
Verifies a batch of proofs with respect to their corresponding vk.
This method does not need to know the Relation the proofs are associated
to and, indeed, it can verify proofs from different Relations.
For that, this function does not take instances, but public inputs
in raw format (Vec<F>).
Returns Ok(()) if all proofs are valid.