pub struct ZkStdLibArch {}Expand description
Architecture of the standard library. Specifies what chips need to be configured.
Fields
jubjub: boolEnable the Jubjub chip?
poseidon: boolEnable the Poseidon chip?
sha2_256: boolEnable the SHA256 chip?
sha2_512: boolEnable the SHA512 chip?
keccak_256: boolEnable the Keccak chip? (third-party implementation)
Note: is configured using the same columns and tables as sha3_256, meaning enabling either of the two, or both, requires the same configuration resources.
sha3_256: boolEnable the Sha3 chip? (third-party implementation)
Note: is configured using the same columns and tables as keccak_256, meaning enabling either of the two, or both, requires the same configuration resources.
blake2b: boolEnable the Blake2b chip? (third-party implementation)
secp256k1: boolEnable the Secp256k1 chip?
p256: boolEnable the P256 chip?
bls12_381: boolEnable BLS12-381 chip?
curve25519: boolEnable Curve25519 chip?
base64: boolEnable base64 chip?
automaton: boolEnable scanner chip (automaton-based parsing and substring checks)?
nr_pow2range_cols: u8Number of parallel lookups for range checks.
Implementations
Sourceimpl ZkStdLibArch
impl ZkStdLibArch
Sourcepub fn write<W: Write>(&self, writer: &mut W) -> Result<()>
pub fn write<W: Write>(&self, writer: &mut W) -> Result<()>
Writes the ZKStd architecture to a buffer.
Sourcepub fn read<R: Read>(reader: &mut R) -> Result<Self>
pub fn read<R: Read>(reader: &mut R) -> Result<Self>
Reads the ZkStd architecture from a buffer.
Sourcepub fn read_from_serialized_vk<R: Read>(reader: &mut R) -> Result<Self>
pub fn read_from_serialized_vk<R: Read>(reader: &mut R) -> Result<Self>
Reads the ZkStdArchitecture from a buffer where a MidnightVK was serialized. This enables the reader to know the architecture without the need of deserializing the full verifying key.